package org.noote.libs.virtualpad;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;

public class VirtualTouchPad_Renderer {

	Bitmap _bitmapVPad;
	Bitmap _bitmapVPad_Stick;
	Bitmap _bitmapVPad_StickPressed;
	Bitmap _bitmapVPad_UpPressed;
	Bitmap _bitmapVPad_DownPressed;
	Bitmap _bitmapVPad_CenterPressed;
	Bitmap _bitmapVPad_LeftPressed;
	Bitmap _bitmapVPad_RightPressed;
	
	public void loadImages(Context ctx, float fScale)
	{
		loadImagesFor5PAD(ctx, fScale);
	}
	
	public void loadImagesFor5PAD(Context ctx, float fScale)
	{
		_bitmapVPad = _load_image_from_resource(ctx, R.drawable.vpad_5pads_transparent, fScale);
		_bitmapVPad_Stick = _load_image_from_resource(ctx, R.drawable.vpad_stick_transparent, fScale);
		_bitmapVPad_StickPressed = _load_image_from_resource(ctx, R.drawable.vpad_stick_pressed_transparent, fScale);
       	_bitmapVPad_UpPressed = _load_image_from_resource(ctx, R.drawable.vpad_5pads_up_pressed_transparent, fScale);
       	_bitmapVPad_DownPressed = _load_image_from_resource(ctx, R.drawable.vpad_5pads_down_pressed_transparent, fScale);
       	_bitmapVPad_CenterPressed = _load_image_from_resource(ctx, R.drawable.vpad_5pads_center_pressed_transparent, fScale);
       	_bitmapVPad_LeftPressed = _load_image_from_resource(ctx, R.drawable.vpad_5pads_left_pressed_transparent, fScale);
       	_bitmapVPad_RightPressed = _load_image_from_resource(ctx, R.drawable.vpad_5pads_right_pressed_transparent, fScale);		
	}

	public int getWidth()
	{
		if(_bitmapVPad != null)
			return _bitmapVPad.getWidth();
		return 0;
	}
	public int getHeight()
	{
		if(_bitmapVPad != null)
			return _bitmapVPad.getHeight();
		return 0;
	}
	public void draw(Canvas canvas, Paint paint, VirtualTouchPad pad)
	{
		if(_bitmapVPad != null)
			canvas.drawBitmap(_bitmapVPad, pad._iMin_X, pad._iMin_Y, paint);
		
		int iSizeX = pad._iMax_X - pad._iMin_X;
		int iSizeY = pad._iMax_Y - pad._iMin_Y;
		int iCenterX = iSizeX/2;
		int iCenterY = iSizeY/2;
		int tCenterStickX = _bitmapVPad_StickPressed.getWidth()/2;
		int iCenterStickY = _bitmapVPad_StickPressed.getHeight()/2;

		if(_bitmapVPad_StickPressed != null && pad.getTouch()>=0.5f)
		{
			canvas.drawBitmap(_bitmapVPad_StickPressed, iCenterX+iSizeX*pad.getX()-tCenterStickX, iCenterY+iSizeY*pad.getY()-iCenterStickY, paint);
		}
		else if(_bitmapVPad_Stick != null && pad.getTouch()<0.5)
		{
			canvas.drawBitmap(_bitmapVPad_Stick, iCenterX+iSizeX*pad.getX()-tCenterStickX, iCenterY+iSizeY*pad.getY()-iCenterStickY, paint);
		}
	}
	public void draw(Canvas canvas, Paint paint, Virtual5DPad pad)
	{
		if(_bitmapVPad != null)
			canvas.drawBitmap(_bitmapVPad, pad._iMin_X, pad._iMin_Y, paint);
		
		if(_bitmapVPad_CenterPressed != null && pad.getButtonCenter())
			canvas.drawBitmap(_bitmapVPad_CenterPressed,pad._iMin_X, pad._iMin_Y, paint);
		
		if(_bitmapVPad_UpPressed != null && pad.getButtonUp())
			canvas.drawBitmap(_bitmapVPad_UpPressed, pad._iMin_X, pad._iMin_Y, paint);
		if(_bitmapVPad_DownPressed != null && pad.getButtonDown())
			canvas.drawBitmap(_bitmapVPad_DownPressed, pad._iMin_X, pad._iMin_Y, paint);
		if(_bitmapVPad_LeftPressed != null && pad.getButtonLeft())
			canvas.drawBitmap(_bitmapVPad_LeftPressed, pad._iMin_X, pad._iMin_Y, paint);
		if(_bitmapVPad_RightPressed != null && pad.getButtonRight())
			canvas.drawBitmap(_bitmapVPad_RightPressed, pad._iMin_X, pad._iMin_Y, paint);
	}
	public void draw(Canvas canvas, Paint paint, Virtual9DPad pad)
	{
		if(_bitmapVPad != null)
			canvas.drawBitmap(_bitmapVPad, pad._iMin_X, pad._iMin_Y, paint);
		
		if(_bitmapVPad_CenterPressed != null && pad.getButtonCenter())
			canvas.drawBitmap(_bitmapVPad_CenterPressed,pad._iMin_X, pad._iMin_Y, paint);
		
		if(_bitmapVPad_UpPressed != null && pad.getButtonUp())
			canvas.drawBitmap(_bitmapVPad_UpPressed, pad._iMin_X, pad._iMin_Y, paint);
		if(_bitmapVPad_DownPressed != null && pad.getButtonDown())
			canvas.drawBitmap(_bitmapVPad_DownPressed, pad._iMin_X, pad._iMin_Y, paint);
		if(_bitmapVPad_LeftPressed != null && pad.getButtonLeft())
			canvas.drawBitmap(_bitmapVPad_LeftPressed, pad._iMin_X, pad._iMin_Y, paint);
		if(_bitmapVPad_RightPressed != null && pad.getButtonRight())
			canvas.drawBitmap(_bitmapVPad_RightPressed, pad._iMin_X, pad._iMin_Y, paint);
		
		if(_bitmapVPad_UpPressed != null && _bitmapVPad_LeftPressed != null && pad.getButtonUpLeft())
		{
			canvas.drawBitmap(_bitmapVPad_UpPressed, pad._iMin_X, pad._iMin_Y, paint);
			canvas.drawBitmap(_bitmapVPad_LeftPressed, pad._iMin_X, pad._iMin_Y, paint);
		}
		if(_bitmapVPad_UpPressed != null && _bitmapVPad_RightPressed != null && pad.getButtonUpRight())
		{
			canvas.drawBitmap(_bitmapVPad_UpPressed, pad._iMin_X, pad._iMin_Y, paint);
			canvas.drawBitmap(_bitmapVPad_RightPressed, pad._iMin_X, pad._iMin_Y, paint);
		}
		if(_bitmapVPad_DownPressed != null && _bitmapVPad_LeftPressed != null && pad.getButtonDownLeft())
		{
			canvas.drawBitmap(_bitmapVPad_DownPressed, pad._iMin_X, pad._iMin_Y, paint);
			canvas.drawBitmap(_bitmapVPad_LeftPressed, pad._iMin_X, pad._iMin_Y, paint);
		}
		if(_bitmapVPad_DownPressed != null && _bitmapVPad_RightPressed != null && pad.getButtonDownRight())
		{
			canvas.drawBitmap(_bitmapVPad_DownPressed, pad._iMin_X, pad._iMin_Y, paint);
			canvas.drawBitmap(_bitmapVPad_RightPressed, pad._iMin_X, pad._iMin_Y, paint);
		}
	}

	private Bitmap _load_image_from_resource(Context ctx, int idRessource, float fScale)
	{
		final Bitmap loadedBitmap = BitmapFactory.decodeStream(ctx.getResources().openRawResource(idRessource));
		if(loadedBitmap != null)
			return Bitmap.createScaledBitmap(loadedBitmap, (int)(loadedBitmap.getWidth()*fScale), (int)(loadedBitmap.getHeight()*fScale), true);
		
		return null;
	}

}
