package org.noote.robot_remote;

import java.text.DecimalFormat;

import org.noote.libs.virtualpad.R;
import org.noote.libs.virtualpad.Virtual5DPad;
import org.noote.libs.virtualpad.VirtualTouchPad_Renderer;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.view.View;

public class DisplayView extends View {
	
	Paint _paint, _paintFont, _paintMessageFont;
	Bitmap _bitmapVTarget;
	
	VirtualTouchPad_Renderer _padRenderer = new VirtualTouchPad_Renderer();
	
	Virtual5DPad _padBody = new Virtual5DPad();
	Virtual5DPad _padTurrel = new Virtual5DPad();
	
	String _sTargetMessage = "";
	boolean _bFrontLeftSwitch = false;
	boolean _bFrontRightSwitch = false;
	boolean _bBackLeftSwitch = false;
	boolean _bBackRightSwitch = false;
	int _iTurrelLow = 0;
	int _iTurrelHigh = 0;
	float _fTurrelDistance = 0.0f;
	boolean _bOnline = false;
	
	static DecimalFormat m_dfDistance = new DecimalFormat("#,##0.00");
	
	public DisplayView(Context context) {
        super(context);
        
        _paint = new Paint();
        _paint.setColor(Color.WHITE);
        _paint.setStyle(Paint.Style.STROKE);
        
        _paintFont = new Paint();
        _paintFont.setColor(Color.WHITE);
        _paintFont.setStyle(Paint.Style.STROKE);
        _paintFont.setTextSize(20);
        _paintFont.setShadowLayer(2, 0, 0, Color.DKGRAY);        

        _paintMessageFont = new Paint();
        _paintMessageFont.setColor(Color.RED);
        _paintMessageFont.setTextSize(32);
        _paintMessageFont.setStyle(Paint.Style.STROKE);
        _paintMessageFont.setShadowLayer(3, 0, 0, Color.WHITE);        
	}
	
	@Override
	protected void onSizeChanged(int width, int height, int width_old, int height_old)
	{
        float fScale = 0.75f;
		if(width>(255*2.5f)) fScale = 1.0f;
        else if(width<255) fScale = 0.5f;
		
       	_bitmapVTarget = _load_image(R.drawable.vtarget_transparent, fScale);
       	
		_padRenderer.loadImages(getContext(), fScale);       	

        _padBody.setWorkSpace(0, height-_padRenderer.getHeight(), _padRenderer.getWidth(), _padRenderer.getHeight());
        _padTurrel.setWorkSpace(width-_padRenderer.getWidth(), height-_padRenderer.getHeight(), _padRenderer.getWidth(), _padRenderer.getHeight());
	}
	
	@Override
	protected void onDraw(Canvas canvas) {
		
		_padRenderer.draw(canvas, _paint, _padBody);
		_padRenderer.draw(canvas, _paint, _padTurrel);
		
		if(_bitmapVTarget != null)
		{
			canvas.drawBitmap(_bitmapVTarget, (getWidth()-_bitmapVTarget.getWidth())/2.0f, (getHeight()-_bitmapVTarget.getHeight())/2.0f, _paint);

			if(_sTargetMessage.length()>0)
				canvas.drawText(_sTargetMessage, getWidth()/2.0f-_paintMessageFont.measureText(_sTargetMessage)/2.0f, getHeight()/2.0f-_bitmapVTarget.getHeight()/2.0f, _paintMessageFont);
			
			String sTurrelDistance = "" + m_dfDistance.format(_fTurrelDistance) + " m";
			canvas.drawText(sTurrelDistance, getWidth()/2.0f-_paintFont.measureText(sTurrelDistance)/2.0f, getHeight()/2.0f+_bitmapVTarget.getHeight()/2.0f+10, _paintFont);
		}
	}
	
	public void setTargetMessag(String sMessage)
	{
		_sTargetMessage = sMessage;
		invalidate();
	}
	
	private Bitmap _load_image(int idRessource, float fScale)
	{
		final Bitmap loadedBitmap = BitmapFactory.decodeStream(getContext().getResources().openRawResource(idRessource));
		if(loadedBitmap != null)
			return Bitmap.createScaledBitmap(loadedBitmap, (int)(loadedBitmap.getWidth()*fScale), (int)(loadedBitmap.getHeight()*fScale), true);
		
		return null;
	}
	
	public Virtual5DPad getBodyPad() {
		return _padBody;
	}

	public Virtual5DPad getTurrelPad() {
		return _padTurrel;
	}
}
