package org.noote.robot_remote;

import org.noote.libs.bluetooth.SPPClient;
import org.noote.libs.virtualpad.Virtual5DPad;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.widget.FrameLayout;
import android.widget.Toast;

public class Remote_Activity extends Activity {

	static final String TAG = "Remote_Activity";
	static final String BT_ADDRESS_DEVICE = "BtAddressDevice";

	SPPClient _SPPClient = new SPPClient();
	
	Virtual5DPad _padTurrel = null;
	Virtual5DPad _padBody = null;

	DisplayView _viewDisplay = null;
	boolean _bOnline = false;
	
	boolean _bMultiTouch = false;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.remote);

        FrameLayout frameDisplay = (FrameLayout) findViewById(R.id.frameDisplay);
        _viewDisplay = new DisplayView(this);
		frameDisplay.addView(_viewDisplay);
		_padBody = _viewDisplay.getBodyPad();
		_padTurrel = _viewDisplay.getTurrelPad();
		_viewDisplay.setOnTouchListener(new OnTouchListener()
		{
			@Override
			public boolean onTouch(View v, MotionEvent event) {
				
				if(_bMultiTouch)
				{
					for (int i = 0; i < event.getPointerCount(); i++) {
						if (event.getAction() == MotionEvent.ACTION_DOWN)
				        {   
							// push somewhere !
							_padBody.setTouchEvent((int)event.getX(i), (int)event.getY(i), true, false);
							_padTurrel.setTouchEvent((int)event.getX(i), (int)event.getY(i), true, false);
				        }
						else if(event.getAction() == MotionEvent.ACTION_MOVE) 
				        {
							// move !
							_padBody.setTouchEvent((int)event.getX(i), (int)event.getY(i), true, true);
							_padTurrel.setTouchEvent((int)event.getX(i), (int)event.getY(i), true, true);
				        }
						else if(event.getAction() == MotionEvent.ACTION_UP) 
				        {
							// release pressure !
							_padBody.setTouchEvent((int)event.getX(i), (int)event.getY(i), false, false);
							_padTurrel.setTouchEvent((int)event.getX(i), (int)event.getY(i), false, false);
				        }
					}
				}
				else
				{
					if (event.getAction() == MotionEvent.ACTION_DOWN)
			        {   
						// push somewhere !
						_padBody.setTouchEvent((int)event.getX(), (int)event.getY(), true, false);
						_padTurrel.setTouchEvent((int)event.getX(), (int)event.getY(), true, false);
			        }
					else if(event.getAction() == MotionEvent.ACTION_MOVE) 
			        {
						// move !
						_padBody.setTouchEvent((int)event.getX(), (int)event.getY(), true, true);
						_padTurrel.setTouchEvent((int)event.getX(), (int)event.getY(), true, true);
			        }
					else if(event.getAction() == MotionEvent.ACTION_UP) 
			        {
						// release pressure !
						_padBody.setTouchEvent((int)event.getX(), (int)event.getY(), false, false);
						_padTurrel.setTouchEvent((int)event.getX(), (int)event.getY(), false, false);
			        }
				}
				
				if(_padBody.haveNewEvent())
				{
					if(_padBody.getTouch()<0.5f || _padBody.getButtonCenter())
					{
						// stop tout mouvement du vehicule !
						_SPPClient.sendMessage("x");
					}
					else
					{
						// ondule ton corps
						if(_padBody.getButtonUp())
							_SPPClient.sendMessage("z"); // go front
						else if(_padBody.getButtonDown())
							_SPPClient.sendMessage("s"); // go back
						else if(_padBody.getButtonLeft())
							_SPPClient.sendMessage("q"); // rotate left
						else if(_padBody.getButtonRight())
							_SPPClient.sendMessage("d"); // rotate right
					}
					_padBody.resetNewEvent();
					_viewDisplay.invalidate();
				}
				if(_padTurrel.haveNewEvent())
				{
					if(_padTurrel.getTouch()<0.5f)
					{
						// stop tout mouvement de la tourelle !
						// nothing todo
					}
					else
					{
						// incremente turrel
						if(_padTurrel.getButtonUp())
							_SPPClient.sendMessage("i");
						else if(_padTurrel.getButtonDown())
							_SPPClient.sendMessage("k");
						else if(_padTurrel.getButtonLeft())
							_SPPClient.sendMessage("j");
						else if(_padTurrel.getButtonRight())
							_SPPClient.sendMessage("l");
						// or reset position !
						else if(_padTurrel.getButtonCenter())
							_SPPClient.sendMessage("r");
					}
					_padTurrel.resetNewEvent();
					_viewDisplay.invalidate();
				}
		        // consume motion event
		        return true;
			}
		});
		Bundle bundle = this.getIntent().getExtras();
		String sAddressDevice = bundle.getString(BT_ADDRESS_DEVICE);
		if(!_SPPClient.connect(sAddressDevice, mBTHandler))
		{
			Toast.makeText(this, "Cannot connect :(", Toast.LENGTH_LONG);    			
		}
    }

	final Handler mBTHandler = new Handler() {
	    @Override
		public void handleMessage(Message msg) {
	        switch (msg.what)
	        {
				case SPPClient.MESSAGE_CONNECTED:
				{
					Toast.makeText(getApplicationContext(), "Connected",
							Toast.LENGTH_SHORT);
					_bOnline = true;
					_SPPClient.sendMessage("v"); // ask version
					_SPPClient.sendMessage("b"); // start rapport sending.
					break;
				}
		        case SPPClient.MESSAGE_DECONNECTED:
		        {
		        	Toast.makeText(getApplicationContext(), "Disconnected", Toast.LENGTH_SHORT);
		        	_bOnline = false;
		            break;
		        }
		        case SPPClient.MESSAGE_GETERROR:
		        {
		        	Toast.makeText(getApplicationContext(), "Get Error : " + (String)msg.obj, Toast.LENGTH_LONG);
		        	break;
		        }
		        case SPPClient.MESSAGE_GETTEXT:
		        {
		        	String sMessage = (String)msg.obj;
	        		String [] sCmds = sMessage.split("\\:");
	        		if(sCmds.length>1)
	        		{
	        			String sCmd = sCmds[0].trim().toLowerCase();
	        			String sOps = sCmds[1].trim().toLowerCase();
			        	if(sCmd.equals("sensors"))
			        	{
			        		String [] sDatas = sOps.split("\\,");
			        		for(String d : sDatas)
			        		{
			        			String [] sVals = d.split("\\=");
			        			String name = sVals[0].trim();
			        			String val = sVals[1].trim();
			        			
			        			if(name.equals("fl")) _viewDisplay._bFrontLeftSwitch = val.equals("1")?true:false;
			        			else if(name.equals("fr")) _viewDisplay._bFrontRightSwitch = val.equals("1")?true:false;
			        			else if(name.equals("bl")) _viewDisplay._bBackLeftSwitch = val.equals("1")?true:false;
			        			else if(name.equals("br")) _viewDisplay._bBackRightSwitch = val.equals("1")?true:false;
			        		}
			        		_viewDisplay.invalidate();
			        	}
			        	else if(sCmd.equals("turrel"))
			        	{
			        		String [] sDatas = sOps.split("\\,");
			        		for(String d : sDatas)
			        		{
			        			String [] sVals = d.split("\\=");
			        			String name = sVals[0].trim();
			        			String val = sVals[1].trim();

			        			if(name.equals("low")) _viewDisplay._iTurrelLow = Integer.parseInt(val);
			        			else if(name.equals("high")) _viewDisplay._iTurrelHigh = Integer.parseInt(val);
			        			else if(name.equals("dist")) _viewDisplay._fTurrelDistance = Float.parseFloat(val);
			        		}
			        		_viewDisplay.invalidate();			        		
			        	}
			        	else
			        		Toast.makeText(getApplicationContext(), "Unknown cmd : " + (String)msg.obj, Toast.LENGTH_LONG);
	        		}
		        	else
		        	{
			        	Toast.makeText(getApplicationContext(), "Receive : " + (String)msg.obj, Toast.LENGTH_LONG);
		        	}
		            break;
		        }
	        }
	    }
	};
}
